@@ -1,30 +1,34 @@
 //! @file a68g-defines.h
 //! @author J. Marcel van der Veer
-//
+//!
 //! @section Copyright
-//
-// This file is part of Algol68G - an Algol 68 compiler-interpreter.
-// Copyright 2001-2022 J. Marcel van der Veer <algol68g@xs4all.nl>.
-//
+//!
+//! This file is part of Algol68G - an Algol 68 compiler-interpreter.
+//! Copyright 2001-2023 J. Marcel van der Veer [algol68g@xs4all.nl].
+//!
 //! @section License
-//
-// This program is free software; you can redistribute it and/or modify it 
-// under the terms of the GNU General Public License as published by the 
-// Free Software Foundation; either version 3 of the License, or 
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
-// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
-// more details. You should have received a copy of the GNU General Public 
-// License along with this program. If not, see <http://www.gnu.org/licenses/>.
+//!
+//! This program is free software; you can redistribute it and/or modify it 
+//! under the terms of the GNU General Public License as published by the 
+//! Free Software Foundation; either version 3 of the License, or 
+//! (at your option) any later version.
+//!
+//! This program is distributed in the hope that it will be useful, but 
+//! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
+//! or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
+//! more details. You should have received a copy of the GNU General Public 
+//! License along with this program. If not, see [http://www.gnu.org/licenses/].
+
+//! @section Synopsis
+//!
+//! General macro definitions.
 
 #if !defined (__A68G_DEFINES_H__)
 #define __A68G_DEFINES_H__
 
 // Constants
 
-#define KILOBYTE ((unsigned) 1024)
+#define KILOBYTE ((unt) 1024)
 #define MEGABYTE (KILOBYTE * KILOBYTE)
 #define GIGABYTE (KILOBYTE * MEGABYTE)
 
@@ -50,17 +54,27 @@
 
 // File extensions
 #define BINARY_EXTENSION ".o"
-#define LIBRARY_EXTENSION ".so"
+#define PLUGIN_EXTENSION ".so"
 #define LISTING_EXTENSION ".l"
 #define OBJECT_EXTENSION ".c"
 #define PRETTY_EXTENSION ".f"
 #define SCRIPT_EXTENSION ""
 
+// Static options for GCC.
+// 
+// -fno-stack-protector is needed for Ubuntu etcetera that enforce -fstack-protector-strong 
+// which may give an undefined reference to `__stack_chk_fail_local'.
+// 
+// -Wno-parentheses-equality is needed for OpenBSD.  
+//
+
+#define A68_GCC_OPTIONS "-DA68_OPTIMISE -ggdb -fno-stack-protector -Wno-parentheses-equality"
+
 // Formats
-#define DIGIT_BLANK ((unsigned) 0x2)
-#define DIGIT_NORMAL ((unsigned) 0x1)
-#define INSERTION_BLANK ((unsigned) 0x20)
-#define INSERTION_NORMAL ((unsigned) 0x10)
+#define DIGIT_BLANK ((unt) 0x2)
+#define DIGIT_NORMAL ((unt) 0x1)
+#define INSERTION_BLANK ((unt) 0x20)
+#define INSERTION_NORMAL ((unt) 0x10)
 
 #define MAX_RESTART 256
 
@@ -89,6 +103,7 @@
 #define NOMADS "></=*"
 #define NOT_EMBEDDED_FORMAT A68_FALSE
 #define NOT_PRINTED 1
+#define OVER_2G(n) ((REAL_T) (n) > (REAL_T) (2 * GIGABYTE)) 
 #define PRIMAL_SCOPE 0
 #define SKIP_PATTERN A68_FALSE
 #define SMALL_BUFFER_SIZE 128
@@ -96,10 +111,18 @@
 #define TRANSPUT_BUFFER_SIZE BUFFER_SIZE
 #define WANT_PATTERN A68_TRUE
 
+#define MANT_DIGS(n) ((int) round ((n) * log10 (2.0)))
+#define MANT_BITS(n) ((int) round ((n) / log10 (2.0)))
+#define REAL_DIGITS MANT_DIGS (REAL_MANT_DIG)
+
 // Macros
 
 #define _SKIP_ { (void) 0;}
 
+#define BUFCLR(z) {memset ((z), 0, BUFFER_SIZE + 1);}
+
+#define ABS(n) ((n) >= 0 ? (n) : -(n))
+
 #define MAX(u, v) (((u) > (v) ? (u) : (v)))
 #define MAXIMISE(u, v) ((u) = MAX (u, v))
 
@@ -152,23 +175,23 @@
   }
 #endif
 
-// Some macros to overcome the ambiguity in having signed or unsigned char 
+// Some macros to overcome the ambiguity in having signed or unt char 
 // on various systems. PDP-11s and IBM 370s are still haunting us with this.
 
-#define IS_ALNUM(c) isalnum ((unsigned char) (c))
-#define IS_ALPHA(c) isalpha ((unsigned char) (c))
-#define IS_CNTRL(c) iscntrl ((unsigned char) (c))
-#define IS_DIGIT(c) isdigit ((unsigned char) (c))
-#define IS_GRAPH(c) isgraph ((unsigned char) (c))
-#define IS_LOWER(c) islower ((unsigned char) (c))
-#define IS_PRINT(c) isprint ((unsigned char) (c))
-#define IS_PUNCT(c) ispunct ((unsigned char) (c))
-#define IS_SPACE(c) isspace ((unsigned char) (c))
-#define IS_UPPER(c) isupper ((unsigned char) (c))
-#define IS_XDIGIT(c) isxdigit ((unsigned char) (c))
-#define TO_LOWER(c) (char) tolower ((unsigned char) (c))
+#define IS_ALNUM(c) isalnum ((unt char) (c))
+#define IS_ALPHA(c) isalpha ((unt char) (c))
+#define IS_CNTRL(c) iscntrl ((unt char) (c))
+#define IS_DIGIT(c) isdigit ((unt char) (c))
+#define IS_GRAPH(c) isgraph ((unt char) (c))
+#define IS_LOWER(c) islower ((unt char) (c))
+#define IS_PRINT(c) isprint ((unt char) (c))
+#define IS_PUNCT(c) ispunct ((unt char) (c))
+#define IS_SPACE(c) isspace ((unt char) (c))
+#define IS_UPPER(c) isupper ((unt char) (c))
+#define IS_XDIGIT(c) isxdigit ((unt char) (c))
+#define TO_LOWER(c) (char) tolower ((unt char) (c))
 #define TO_UCHAR(c) ((c) >= 0 ? (int) (c) : (int) (UCHAR_MAX + (int) (c) + 1))
-#define TO_UPPER(c) (char) toupper ((unsigned char) (c))
+#define TO_UPPER(c) (char) toupper ((unt char) (c))
 
 // Macro's for fat A68 pointers
 
@@ -200,12 +223,8 @@
 #define A68_REF_SIZE (SIZE_ALIGNED (A68_REF))
 #define A68_UNION_SIZE (SIZE_ALIGNED (A68_UNION))
 
-#define LINK_ERRNO int _errnol_ = errno
-#define UNLINK_ERRNO errno = _errnol_
-
 #define A68_SOUND_BYTES(s) ((int) (BITS_PER_SAMPLE (s)) / 8 + (int) (BITS_PER_SAMPLE (s) % 8 == 0 ? 0 : 1))
 #define A68_SOUND_DATA_SIZE(s) ((int) (NUM_SAMPLES (s)) * (int) (NUM_CHANNELS (s)) * (int) (A68_SOUND_BYTES (s)))
-#define ABS(n) ((n) >= 0 ? (n) : -(n))
 #define BACKWARD(p) (p = PREVIOUS (p))
 #define BITS_WIDTH ((int) (1 + ceil (log ((REAL_T) A68_MAX_INT) / log ((REAL_T) 2))))
 #define DEFLEX(p) (DEFLEXED (p) != NO_MOID ? DEFLEXED(p) : (p))
@@ -214,9 +233,6 @@
 #define LONG_INT_WIDTH (1 + LONG_WIDTH)
 #define PM(m) (moid_to_string (m, 132, NO_NODE))
 #define SIGN(n) ((n) == 0 ? 0 : ((n) > 0 ? 1 : -1))
-#define STATUS_CLEAR(p, q) {STATUS (p) &= (~(q));}
-#define STATUS_SET(p, q) {STATUS (p) |= (q);}
-#define STATUS_TEST(p, q) ((STATUS (p) & (q)) != (unsigned) 0)
 #define WIS(p) where_in_source (STDOUT_FILENO, (p))
 #define WRITE(f, s) io_write_string ((f), (s));
 #define WRITELN(f, s) {WRITE ((f), "\n"); WRITE ((f), (s));}
@@ -274,7 +290,7 @@
 #define DEVICE_OPENED(p) ((p)->device_opened)
 #define DIAGNOSTICS(p) ((p)->diagnostics)
 #define DIGITS(p) ((p)->digits)
-#define DIGITSC(p) ((p)->digitsc)
+#define DIGITS_COMPL(p) ((p)->digits_compl)
 #define DIM(p) ((p)->dim)
 #define DISPLAY(p) ((p)->display)
 #define DRAW(p) ((p)->draw)
@@ -309,9 +325,9 @@
 #define FILE_ENTRY(p) ((p)->file_entry)
 #define FILE_GENERIC_NAME(p) (FILES (p).generic_name)
 #define FILE_INITIAL_NAME(p) (FILES (p).initial_name)
-#define FILE_LIBRARY_NAME(p) (FILES (p).library.name)
-#define FILE_LIBRARY_OPENED(p) (FILES (p).library.opened)
-#define FILE_LIBRARY_WRITEMOOD(p) (FILES (p).library.writemood)
+#define FILE_PLUGIN_NAME(p) (FILES (p).plugin.name)
+#define FILE_PLUGIN_OPENED(p) (FILES (p).plugin.opened)
+#define FILE_PLUGIN_WRITEMOOD(p) (FILES (p).plugin.writemood)
 #define FILE_LISTING_FD(p) (FILES (p).listing.fd)
 #define FILE_LISTING_NAME(p) (FILES (p).listing.name)
 #define FILE_LISTING_OPENED(p) (FILES (p).listing.opened)
@@ -537,6 +553,7 @@
 #define SCALE_ROW(p) ((p)->scale_row)
 #define SCAN(p) ((p)->scan)
 #define SCAN_ERROR(c, u, v, txt) if (c) {scan_error (u, v, txt);}
+#define SCAN_WARNING(c, u, v, txt) if (c) {scan_warning (u, v, txt);}
 #define SCOPE(p) ((p)->scope)
 #define SCOPE_ASSIGNED(p) ((p)->scope_assigned)
 #define SEARCH(p) ((p)->search)
@@ -551,7 +568,7 @@
 #define SIZE(p) ((p)->size)
 #define SIZE1(p) ((p)->size1)
 #define SIZE2(p) ((p)->size2)
-#define SIZEC(p) ((p)->sizec)
+#define SIZE_COMPL(p) ((p)->size_compl)
 #define SLICE(p) ((p)->slice)
 #define SLICE_OFFSET(p) ((p)->slice_offset)
 #define SO(p) ((p)->so)
